﻿namespace ProjectPlane.CollectionGenericObjects;

public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    private T?[] _collection;
    public int Count => _collection.Length;
    public int SetMaxCount
    {
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }
    public T? Get(int position)
    {
        if (position < 0 || position > Count)
        {
            return null;
        }
        return _collection[position];
    }

    public int Insert(T obj)
    {
        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }
        return -1;
    }
    public int Insert(T obj, int position)
    {
        if (position < 0 || position > Count)
        {
            return -1;
        }

        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }

        for (int i = position + 1; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return position;
            }
        }

        for (int i = position - 1; i >= 0; i--)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return position;
            }
        }

        return -1;
    }
    public T? Remove(int position)
    {
        if (position < 0 || position > Count || _collection[position] == null)
        {
            return null;
        }

        T? obj = _collection[position];
        _collection[position] = null;
        return obj;
    }

}
