﻿
using ProjectPlane.CollectionGenericObjects;
using ProjectPlane.Drawnings;

namespace ProjectPlane;

public partial class FormShipCollection : Form
{
    private AbstractCompany? _company = null;

    public FormShipCollection()
    {
        InitializeComponent();
    }

    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        switch (ComboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new Marina(pictureBox.Width, pictureBox.Height, new MassiveGenericObjects<DrawningShip>());
                break;
        }
    }

    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }


        Random random = new Random();
        DrawningShip drawShip;
        switch (type)
        {
            case nameof(DrawningShip):
                drawShip = new DrawningShip(random.Next(100, 300), random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawCont):
                drawShip = new DrawCont(random.Next(100, 300), random.Next(1000, 3000),
                    GetColor(random), GetColor(random), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        if (_company + drawShip != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }

    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }
        return color;
    }

    private void ButtonAddShip_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningShip));

    private void ButtonAddCont_Click(object sender, EventArgs e) => CreateObject(nameof(DrawCont));

    private void ButtonDel_Click(object sender, EventArgs e)
    {

        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удалён");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }

    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningShip? ship = null;
        int counter = 100;
        while (ship == null)
        {
            ship = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }

        if (ship == null)
        {
            return;
        }

        FormContainer form = new()
        {
            SetShip = ship
        };
        form.ShowDialog();

    }

    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }
}
