﻿namespace ProjectPlane.MovementStrategy;

public abstract class AbstractStrategy
{
    private IMoveableObject? _moveableObject;

    private StrategyStatus _state = StrategyStatus.NotInit;

    protected int FieldWidth { get; private set; }

    protected int FieldHeight { get; private set; }

    public StrategyStatus GetStatus() { return _state; }

    public void SetData(IMoveableObject moveableObject, int width, int height)
    {
        if (moveableObject == null)
        {
            _state = StrategyStatus.NotInit;
            return;
        }
        _state = StrategyStatus.InProgress;
        _moveableObject = moveableObject;
        FieldWidth = width;
        FieldHeight = height;
    }

    public void MakeStep()
    {
        if (_state != StrategyStatus.InProgress)
        {
            return;
        }
        if (IsTargetDestinaion())
        {
            _state = StrategyStatus.Finish;
            return;
        }
        MoveToTarget();
    }


    protected bool MoveLeft() => MoveTo(MovementDirection.Left);

    protected bool MoveRight() => MoveTo(MovementDirection.Right);

    protected bool MoveUp() => MoveTo(MovementDirection.Up);

    protected bool MoveDown() => MoveTo(MovementDirection.Down);

    protected ObjectParameters? GetObjectParameters => _moveableObject?.GetObjectPosition;

    protected int? GetStep()
    {
        if (_state != StrategyStatus.InProgress)
        {
            return null;
        }
        return _moveableObject?.GetStep;
    }

    protected abstract void MoveToTarget();

    protected abstract bool IsTargetDestinaion();

    private bool MoveTo(MovementDirection movementDirection)
    {
        if (_state != StrategyStatus.InProgress)
        {
            return false;
        }
        return _moveableObject?.TryMoveObject(movementDirection) ?? false;
    }
}
