﻿using ProjectPlane.Drawnings;

namespace ProjectPlane.MovementStrategy;

public class MoveableShip : IMoveableObject
{
    private readonly DrawningShip? _ship = null;

    public MoveableShip(DrawningShip ship)
    {
        _ship = ship;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_ship == null || _ship.EntityShip == null || !_ship.GetPosX.HasValue || !_ship.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_ship.GetPosX.Value, _ship.GetPosY.Value, _ship.GetWidth, _ship.GetHeight);
        }
    }

    public int GetStep => (int)(_ship?.EntityShip?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_ship == null || _ship.EntityShip == null)
        {
            return false;
        }
        return _ship.MoveTransport(GetDirectionType(direction));
    }

    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }

}
